@@ -1,27 +1,42 @@
 DEFINE_TEST(test_0)
 DEFINE_TEST(test_basic)
 DEFINE_TEST(test_cmdline)
+DEFINE_TEST(test_extract_cpio_bz2)
+DEFINE_TEST(test_extract_cpio_grz)
+DEFINE_TEST(test_extract_cpio_gz)
+DEFINE_TEST(test_extract_cpio_lrz)
+DEFINE_TEST(test_extract_cpio_lz)
+DEFINE_TEST(test_extract_cpio_lzma)
+DEFINE_TEST(test_extract_cpio_lzo)
+DEFINE_TEST(test_extract_cpio_xz)
+DEFINE_TEST(test_extract_cpio_Z)
 DEFINE_TEST(test_format_newc)
 DEFINE_TEST(test_gcpio_compat)
 DEFINE_TEST(test_option_0)
-DEFINE_TEST(test_option_B_upper)
-DEFINE_TEST(test_option_C_upper)
-DEFINE_TEST(test_option_J_upper)
-DEFINE_TEST(test_option_L_upper)
-DEFINE_TEST(test_option_Z_upper)
 DEFINE_TEST(test_option_a)
+DEFINE_TEST(test_option_b64encode)
+DEFINE_TEST(test_option_B_upper)
 DEFINE_TEST(test_option_c)
+DEFINE_TEST(test_option_C_upper)
 DEFINE_TEST(test_option_d)
 DEFINE_TEST(test_option_f)
+DEFINE_TEST(test_option_grzip)
 DEFINE_TEST(test_option_help)
+DEFINE_TEST(test_option_J_upper)
 DEFINE_TEST(test_option_l)
+DEFINE_TEST(test_option_lrzip)
+DEFINE_TEST(test_option_L_upper)
 DEFINE_TEST(test_option_lzma)
+DEFINE_TEST(test_option_lzop)
 DEFINE_TEST(test_option_m)
 DEFINE_TEST(test_option_t)
 DEFINE_TEST(test_option_u)
+DEFINE_TEST(test_option_uuencode)
 DEFINE_TEST(test_option_version)
+DEFINE_TEST(test_option_xz)
 DEFINE_TEST(test_option_y)
 DEFINE_TEST(test_option_z)
+DEFINE_TEST(test_option_Z_upper)
 DEFINE_TEST(test_owner_parse)
 DEFINE_TEST(test_passthrough_dotdot)
 DEFINE_TEST(test_passthrough_reverse)
