@@ -87,6 +87,8 @@ _archive_set_either_option(struct archive *a, const char *m, const char *o, cons
 	if (r2 == ARCHIVE_FATAL)
 		return (ARCHIVE_FATAL);
 
+	if (r2 == ARCHIVE_WARN - 1)
+		return r1;
 	return r1 > r2 ? r1 : r2;
 }
 
@@ -94,7 +96,7 @@ int
 _archive_set_options(struct archive *a, const char *options,
     int magic, const char *fn, option_handler use_option)
 {
-	int allok = 1, anyok = 0, r;
+	int allok = 1, anyok = 0, ignore_mod_err = 0, r;
 	char *data;
 	const char *s, *mod, *opt, *val;
 
@@ -111,6 +113,15 @@ _archive_set_options(struct archive *a, const char *options,
 		mod = opt = val = NULL;
 
 		parse_option(&s, &mod, &opt, &val);
+		if (mod == NULL && opt != NULL &&
+		    strcmp("__ignore_wrong_module_name__", opt) == 0) {
+			/* Ignore module name error */
+			if (val != NULL) {
+				ignore_mod_err = 1;
+				anyok = 1;
+			}
+			continue;
+		}
 
 		r = use_option(a, mod, opt, val);
 		if (r == ARCHIVE_FATAL) {
@@ -122,6 +133,8 @@ _archive_set_options(struct archive *a, const char *options,
 			return (ARCHIVE_FAILED);
 		}
 		if (r == ARCHIVE_WARN - 1) {
+			if (ignore_mod_err)
+				continue;
 			/* The module name is wrong. */
 			archive_set_error(a, ARCHIVE_ERRNO_MISC,
 			    "Unknown module name: `%s'", mod);
