@@ -201,6 +201,7 @@ struct lzss
 
 struct data_block_offsets
 {
+  int64_t header_size;
   int64_t start_offset;
   int64_t end_offset;
 };
@@ -241,6 +242,7 @@ struct rar
   int64_t bytes_uncopied;
   int64_t offset;
   int64_t offset_outgoing;
+  int64_t offset_seek;
   char valid;
   unsigned int unp_offset;
   unsigned int unp_buffer_size;
@@ -942,10 +944,12 @@ archive_read_format_rar_read_data(struct archive_read *a, const void **buff,
       rar->bytes_unconsumed = 0;
   }
 
-  if (rar->entry_eof) {
+  if (rar->entry_eof || rar->offset_seek >= rar->unp_size) {
     *buff = NULL;
     *size = 0;
     *offset = rar->offset;
+    if (*offset < rar->unp_size)
+      *offset = rar->unp_size;
     return (ARCHIVE_EOF);
   }
 
@@ -1019,66 +1023,126 @@ archive_read_format_rar_seek_data(struct archive_read *a, int64_t offset,
   unsigned int i;
   struct rar *rar = (struct rar *)(a->format->data);
 
-  if (rar->compression_method & COMPRESS_METHOD_STORE)
+  if (rar->compression_method == COMPRESS_METHOD_STORE)
   {
     /* Modify the offset for use with SEEK_SET */
     switch (whence)
     {
-      case SEEK_SET:
-        break;
-
       case SEEK_CUR:
-        offset += rar->offset;
+        client_offset = rar->offset_seek;
         break;
-
       case SEEK_END:
-        offset += rar->unp_size - 1;
+        client_offset = rar->unp_size;
         break;
+      case SEEK_SET:
+      default:
+        client_offset = 0;
+    }
+    client_offset += offset;
+    if (client_offset < 0)
+    {
+      /* Can't seek past beginning of data block */
+      return -1;
+    }
+    else if (client_offset > rar->unp_size)
+    {
+      /*
+       * Set the returned offset but only seek to the end of
+       * the data block.
+       */
+      rar->offset_seek = client_offset;
+      client_offset = rar->unp_size;
     }
-    if (offset < 0)
-      offset = 0;
-    else if (offset > rar->unp_size - 1)
-      offset = rar->unp_size - 1;
 
-    /* Find the appropriate offset in the client */
+    client_offset += rar->dbo[0].start_offset;
     i = 0;
-    client_offset = offset;
-    client_offset += rar->dbo[i].start_offset;
-    while (rar->main_flags & MHD_VOLUME &&
-      client_offset > rar->dbo[i].end_offset)
+    while (i < rar->cursor)
     {
       i++;
-      if (i >= rar->nodes || client_offset <= rar->dbo[i].end_offset)
+      client_offset += rar->dbo[i].start_offset - rar->dbo[i-1].end_offset;
+    }
+    if (rar->main_flags & MHD_VOLUME)
+    {
+      /* Find the appropriate offset among the multivolume archive */
+      while (1)
       {
-        ret = __archive_read_seek(a, rar->dbo[i-1].end_offset + 1, SEEK_SET);
-        if (ret < (ARCHIVE_OK))
-          return ret;
-        ret = archive_read_format_rar_read_header(a, a->entry);
-        if (ret == (ARCHIVE_EOF))
+        if (client_offset < rar->dbo[rar->cursor].start_offset &&
+          rar->file_flags & FHD_SPLIT_BEFORE)
         {
-          rar->has_endarc_header = 1;
+          /* Search backwards for the correct data block */
+          if (rar->cursor == 0)
+          {
+            archive_set_error(&a->archive, ARCHIVE_ERRNO_MISC,
+              "Attempt to seek past beginning of RAR data block");
+            return (ARCHIVE_FAILED);
+          }
+          rar->cursor--;
+          client_offset -= rar->dbo[rar->cursor+1].start_offset -
+            rar->dbo[rar->cursor].end_offset;
+          if (client_offset < rar->dbo[rar->cursor].start_offset)
+            continue;
+          ret = __archive_read_seek(a, rar->dbo[rar->cursor].start_offset -
+            rar->dbo[rar->cursor].header_size, SEEK_SET);
+          if (ret < (ARCHIVE_OK))
+            return ret;
           ret = archive_read_format_rar_read_header(a, a->entry);
+          if (ret != (ARCHIVE_OK))
+          {
+            archive_set_error(&a->archive, ARCHIVE_ERRNO_MISC,
+              "Error during seek of RAR file");
+            return (ARCHIVE_FAILED);
+          }
+          rar->cursor--;
+          break;
         }
-        if (ret != (ARCHIVE_OK))
+        else if (client_offset > rar->dbo[rar->cursor].end_offset &&
+          rar->file_flags & FHD_SPLIT_AFTER)
         {
-          archive_set_error(&a->archive, ARCHIVE_ERRNO_MISC,
-            "Error during seek of RAR file");
-          return (ARCHIVE_FAILED);
+          /* Search forward for the correct data block */
+          rar->cursor++;
+          if (rar->cursor < rar->nodes &&
+            client_offset > rar->dbo[rar->cursor].end_offset)
+          {
+            client_offset += rar->dbo[rar->cursor].start_offset -
+              rar->dbo[rar->cursor-1].end_offset;
+            continue;
+          }
+          rar->cursor--;
+          ret = __archive_read_seek(a, rar->dbo[rar->cursor].end_offset,
+                                    SEEK_SET);
+          if (ret < (ARCHIVE_OK))
+            return ret;
+          ret = archive_read_format_rar_read_header(a, a->entry);
+          if (ret == (ARCHIVE_EOF))
+          {
+            rar->has_endarc_header = 1;
+            ret = archive_read_format_rar_read_header(a, a->entry);
+          }
+          if (ret != (ARCHIVE_OK))
+          {
+            archive_set_error(&a->archive, ARCHIVE_ERRNO_MISC,
+              "Error during seek of RAR file");
+            return (ARCHIVE_FAILED);
+          }
+          client_offset += rar->dbo[rar->cursor].start_offset -
+            rar->dbo[rar->cursor-1].end_offset;
+          continue;
         }
+        break;
       }
-      client_offset += rar->dbo[i].start_offset - rar->dbo[i-1].end_offset - 1;
     }
 
     ret = __archive_read_seek(a, client_offset, SEEK_SET);
     if (ret < (ARCHIVE_OK))
       return ret;
-    rar->bytes_remaining = rar->dbo[i].end_offset - ret + 1;
+    rar->bytes_remaining = rar->dbo[rar->cursor].end_offset - ret;
+    i = rar->cursor;
     while (i > 0)
     {
-      ret -= rar->dbo[i].start_offset - rar->dbo[i-1].end_offset - 1;
       i--;
+      ret -= rar->dbo[i+1].start_offset - rar->dbo[i].end_offset;
     }
-    ret -= rar->dbo[i].start_offset;
+    ret -= rar->dbo[0].start_offset;
 
     /* Always restart reading the file after a seek */
     a->read_data_block = NULL;
@@ -1088,7 +1152,16 @@ archive_read_format_rar_seek_data(struct archive_read *a, int64_t offset,
     rar->bytes_unconsumed = 0;
     rar->offset = 0;
 
-    return ret;
+    /*
+     * If a seek past the end of file was requested, return the requested
+     * offset.
+     */
+    if (ret == rar->unp_size && rar->offset_seek > rar->unp_size)
+      return rar->offset_seek;
+
+    /* Return the new offset */
+    rar->offset_seek = ret;
+    return rar->offset_seek;
   }
   else
   {
@@ -1409,6 +1482,7 @@ read_header(struct archive_read *a, struct archive_entry *entry,
         archive_set_error(&a->archive, ENOMEM, "Couldn't allocate memory.");
         return (ARCHIVE_FATAL);
       }
+      rar->dbo[rar->cursor].header_size = header_size;
       rar->dbo[rar->cursor].start_offset = -1;
       rar->dbo[rar->cursor].end_offset = -1;
     }
@@ -1416,7 +1490,7 @@ read_header(struct archive_read *a, struct archive_entry *entry,
     {
       rar->dbo[rar->cursor].start_offset = a->filter->position;
       rar->dbo[rar->cursor].end_offset = rar->dbo[rar->cursor].start_offset +
-        rar->packed_size - 1;
+        rar->packed_size;
     }
     return ret;
   }
@@ -1432,6 +1506,7 @@ read_header(struct archive_read *a, struct archive_entry *entry,
     archive_set_error(&a->archive, ENOMEM, "Couldn't allocate memory.");
     return (ARCHIVE_FATAL);
   }
+  rar->dbo[0].header_size = header_size;
   rar->dbo[0].start_offset = -1;
   rar->dbo[0].end_offset = -1;
   rar->cursor = 0;
@@ -1458,7 +1533,7 @@ read_header(struct archive_read *a, struct archive_entry *entry,
 
   __archive_read_consume(a, header_size - 7);
   rar->dbo[0].start_offset = a->filter->position;
-  rar->dbo[0].end_offset = rar->dbo[0].start_offset + rar->packed_size - 1;
+  rar->dbo[0].end_offset = rar->dbo[0].start_offset + rar->packed_size;
 
   switch(file_header.host_os)
   {
@@ -1487,6 +1562,7 @@ read_header(struct archive_read *a, struct archive_entry *entry,
 
   rar->bytes_uncopied = rar->bytes_unconsumed = 0;
   rar->lzss.position = rar->offset = 0;
+  rar->offset_seek = 0;
   rar->dictionary_size = 0;
   rar->offset_outgoing = 0;
   rar->br.cache_avail = 0;
@@ -1698,6 +1774,7 @@ read_data_stored(struct archive_read *a, const void **buff, size_t *size,
   *size = bytes_avail;
   *offset = rar->offset;
   rar->offset += bytes_avail;
+  rar->offset_seek += bytes_avail;
   rar->bytes_remaining -= bytes_avail;
   rar->bytes_unconsumed = bytes_avail;
   /* Calculate File CRC. */
@@ -2757,6 +2834,8 @@ rar_read_ahead(struct archive_read *a, size_t min, ssize_t *avail)
   int ret;
   if (avail)
   {
+    if (a->read_data_is_posix_read && *avail > (ssize_t)a->read_data_requested)
+      *avail = a->read_data_requested;
     if (*avail > rar->bytes_remaining)
       *avail = (ssize_t)rar->bytes_remaining;
     if (*avail < 0)
