@@ -385,7 +385,7 @@ _7z_options(struct archive_write *a, const char *key, const char *value)
 		else {
 			archive_set_error(&(a->archive),
 			    ARCHIVE_ERRNO_MISC,
-			    "Unkonwn compression name: `%s'",
+			    "Unknown compression name: `%s'",
 			    value);
 			return (ARCHIVE_FAILED);
 		}
@@ -405,7 +405,7 @@ _7z_options(struct archive_write *a, const char *key, const char *value)
 		    value[1] != '\0') {
 			archive_set_error(&(a->archive),
 			    ARCHIVE_ERRNO_MISC,
-			    "Illeagal value `%s'",
+			    "Illegal value `%s'",
 			    value);
 			return (ARCHIVE_FAILED);
 		}
@@ -442,6 +442,14 @@ _7z_write_header(struct archive_write *a, struct archive_entry *entry)
 		file_free(file);
 		return (r);
 	}
+	if (file->size == 0 && file->dir) {
+		if (!__archive_rb_tree_insert_node(&(zip->rbtree),
+		    (struct archive_rb_node *)file)) {
+			/* We have already had the same file. */
+			file_free(file);
+			return (ARCHIVE_OK);
+		}
+	}
 
 	if (file->flg & MTIME_IS_SET)
 		zip->total_number_time_defined[MTIME]++;
@@ -450,11 +458,6 @@ _7z_write_header(struct archive_write *a, struct archive_entry *entry)
 	if (file->flg & ATIME_IS_SET)
 		zip->total_number_time_defined[ATIME]++;
 
-	if (file->size == 0 && file->dir) {
-		if (!__archive_rb_tree_insert_node(&(zip->rbtree),
-		    (struct archive_rb_node *)file))
-			file_free(file);
-	}
 	zip->total_number_entry++;
 	zip->total_bytes_entry_name += file->name_len + 2;
 	if (file->size == 0) {
@@ -501,7 +504,7 @@ _7z_write_header(struct archive_write *a, struct archive_entry *entry)
 		bytes = compress_out(a, p, (size_t)file->size, ARCHIVE_Z_RUN);
 		if (bytes < 0)
 			return ((int)bytes);
-		zip->entry_crc32 = crc32(zip->entry_crc32, p, bytes);
+		zip->entry_crc32 = crc32(zip->entry_crc32, p, (unsigned)bytes);
 		zip->entry_bytes_remaining -= bytes;
 	}
 
@@ -559,10 +562,11 @@ compress_out(struct archive_write *a, const void *buff, size_t s,
 		return (0);
 
 	if ((zip->crc32flg & PRECODE_CRC32) && s)
-		zip->precode_crc32 = crc32(zip->precode_crc32, buff, s);
+		zip->precode_crc32 = crc32(zip->precode_crc32, buff,
+		    (unsigned)s);
 	zip->stream.next_in = (const unsigned char *)buff;
 	zip->stream.avail_in = s;
-	do {
+	for (;;) {
 		/* Compress file data. */
 		r = compression_code(&(a->archive), &(zip->stream), run);
 		if (r != ARCHIVE_OK && r != ARCHIVE_EOF)
@@ -576,8 +580,12 @@ compress_out(struct archive_write *a, const void *buff, size_t s,
 			if (zip->crc32flg & ENCODED_CRC32)
 				zip->encoded_crc32 = crc32(zip->encoded_crc32,
 				    zip->wbuff, sizeof(zip->wbuff));
+			if (run == ARCHIVE_Z_FINISH && r != ARCHIVE_EOF)
+				continue;
 		}
-	} while (zip->stream.avail_in);
+		if (zip->stream.avail_in == 0)
+			break;
+	}
 	if (run == ARCHIVE_Z_FINISH) {
 		uint64_t bytes = sizeof(zip->wbuff) - zip->stream.avail_out;
 		if (write_to_temp(a, zip->wbuff, (size_t)bytes) != ARCHIVE_OK)
@@ -605,7 +613,7 @@ _7z_write_data(struct archive_write *a, const void *buff, size_t s)
 	bytes = compress_out(a, buff, s, ARCHIVE_Z_RUN);
 	if (bytes < 0)
 		return (bytes);
-	zip->entry_crc32 = crc32(zip->entry_crc32, buff, bytes);
+	zip->entry_crc32 = crc32(zip->entry_crc32, buff, (unsigned)bytes);
 	zip->entry_bytes_remaining -= bytes;
 	return (bytes);
 }
@@ -627,7 +635,7 @@ _7z_finish_entry(struct archive_write *a)
 			s = a->null_length;
 		r = _7z_write_data(a, a->nulls, s);
 		if (r < 0)
-			return (r);
+			return ((int)r);
 	}
 	zip->total_bytes_compressed += zip->stream.total_in;
 	zip->total_bytes_uncompressed += zip->stream.total_out;
@@ -859,7 +867,7 @@ enc_uint64(struct archive_write *a, uint64_t val)
 		numdata[0] |= mask;
 		mask >>= 1;
 	}
-	return (compress_out(a, numdata, i, ARCHIVE_Z_RUN));
+	return ((int)compress_out(a, numdata, i, ARCHIVE_Z_RUN));
 }
 
 static int
@@ -924,7 +932,7 @@ make_substreamsInfo(struct archive_write *a, struct coder *coders)
 		if (file->size == 0)
 			break;
 		archive_le32enc(crc, file->crc32);
-		r = compress_out(a, crc, 4, ARCHIVE_Z_RUN);
+		r = (int)compress_out(a, crc, 4, ARCHIVE_Z_RUN);
 		if (r < 0)
 			return (r);
 	}
@@ -948,7 +956,7 @@ make_streamsInfo(struct archive_write *a, uint64_t offset, uint64_t pack_size,
 	int i, r;
 
 	if (coders->codec == _7Z_COPY)
-		numFolders = zip->total_number_nonempty_entry;
+		numFolders = (int)zip->total_number_nonempty_entry;
 	else
 		numFolders = 1;
 
@@ -1044,7 +1052,7 @@ make_streamsInfo(struct archive_write *a, uint64_t offset, uint64_t pack_size,
 
 			/* Write Codec ID. */
 			codec_size &= 0x0f;
-			r = compress_out(a, &codec_buff[8-codec_size],
+			r = (int)compress_out(a, &codec_buff[8-codec_size],
 				codec_size, ARCHIVE_Z_RUN);
 			if (r < 0)
 				return (r);
@@ -1056,7 +1064,7 @@ make_streamsInfo(struct archive_write *a, uint64_t offset, uint64_t pack_size,
 					return (r);
 
 				/* Write Codec properties. */
-				r = compress_out(a, coders[i].props,
+				r = (int)compress_out(a, coders[i].props,
 					coders[i].prop_size, ARCHIVE_Z_RUN);
 				if (r < 0)
 					return (r);
@@ -1102,7 +1110,7 @@ make_streamsInfo(struct archive_write *a, uint64_t offset, uint64_t pack_size,
 		if (r < 0)
 			return (r);
 		archive_le32enc(crc, header_crc);
-		r = compress_out(a, crc, 4, ARCHIVE_Z_RUN);
+		r = (int)compress_out(a, crc, 4, ARCHIVE_Z_RUN);
 		if (r < 0)
 			return (r);
 	}
@@ -1196,7 +1204,7 @@ make_time(struct archive_write *a, uint8_t type, unsigned flg, int ti)
 				b |= mask;
 			mask >>= 1;
 			if (mask == 0) {
-				r = compress_out(a, &b, 1, ARCHIVE_Z_RUN);
+				r = (int)compress_out(a, &b, 1, ARCHIVE_Z_RUN);
 				if (r < 0)
 					return (r);
 				mask = 0x80;
@@ -1204,7 +1212,7 @@ make_time(struct archive_write *a, uint8_t type, unsigned flg, int ti)
 			}
 		}
 		if (mask != 0x80) {
-			r = compress_out(a, &b, 1, ARCHIVE_Z_RUN);
+			r = (int)compress_out(a, &b, 1, ARCHIVE_Z_RUN);
 			if (r < 0)
 				return (r);
 		}
@@ -1225,7 +1233,7 @@ make_time(struct archive_write *a, uint8_t type, unsigned flg, int ti)
 			continue;
 		archive_le64enc(filetime, utcToFiletime(file->times[ti].time,
 			file->times[ti].time_ns));
-		r = compress_out(a, filetime, 8, ARCHIVE_Z_RUN);
+		r = (int)compress_out(a, filetime, 8, ARCHIVE_Z_RUN);
 		if (r < 0)
 			return (r);
 	}
@@ -1296,7 +1304,7 @@ make_header(struct archive_write *a, uint64_t offset, uint64_t pack_size,
 				b |= mask;
 			mask >>= 1;
 			if (mask == 0) {
-				r = compress_out(a, &b, 1, ARCHIVE_Z_RUN);
+				r = (int)compress_out(a, &b, 1, ARCHIVE_Z_RUN);
 				if (r < 0)
 					return (r);
 				mask = 0x80;
@@ -1304,7 +1312,7 @@ make_header(struct archive_write *a, uint64_t offset, uint64_t pack_size,
 			}
 		}
 		if (mask != 0x80) {
-			r = compress_out(a, &b, 1, ARCHIVE_Z_RUN);
+			r = (int)compress_out(a, &b, 1, ARCHIVE_Z_RUN);
 			if (r < 0)
 				return (r);
 		}
@@ -1331,7 +1339,7 @@ make_header(struct archive_write *a, uint64_t offset, uint64_t pack_size,
 				b |= mask;
 			mask >>= 1;
 			if (mask == 0) {
-				r = compress_out(a, &b, 1, ARCHIVE_Z_RUN);
+				r = (int)compress_out(a, &b, 1, ARCHIVE_Z_RUN);
 				if (r < 0)
 					return (r);
 				mask = 0x80;
@@ -1339,7 +1347,7 @@ make_header(struct archive_write *a, uint64_t offset, uint64_t pack_size,
 			}
 		}
 		if (mask != 0x80) {
-			r = compress_out(a, &b, 1, ARCHIVE_Z_RUN);
+			r = (int)compress_out(a, &b, 1, ARCHIVE_Z_RUN);
 			if (r < 0)
 				return (r);
 		}
@@ -1362,7 +1370,7 @@ make_header(struct archive_write *a, uint64_t offset, uint64_t pack_size,
 
 	file = zip->file_list.first;
 	for (;file != NULL; file = file->next) {
-		r = compress_out(a, file->utf16name, file->name_len+2,
+		r = (int)compress_out(a, file->utf16name, file->name_len+2,
 			ARCHIVE_Z_RUN);
 		if (r < 0)
 			return (r);
@@ -1418,7 +1426,7 @@ make_header(struct archive_write *a, uint64_t offset, uint64_t pack_size,
 			attr |= 1;/* Read Only. */
 		attr |= ((uint32_t)file->mode) << 16;
 		archive_le32enc(&encattr, attr);
-		r = compress_out(a, &encattr, 4, ARCHIVE_Z_RUN);
+		r = (int)compress_out(a, &encattr, 4, ARCHIVE_Z_RUN);
 		if (r < 0)
 			return (r);
 	}
@@ -1512,7 +1520,7 @@ file_new(struct archive_write *a, struct archive_entry *entry,
 	memcpy(file->utf16name, u16, u16len);
 	file->utf16name[u16len+0] = 0;
 	file->utf16name[u16len+1] = 0;
-	file->name_len = u16len;
+	file->name_len = (unsigned)u16len;
 	file->mode = archive_entry_mode(entry);
 	if (archive_entry_filetype(entry) == AE_IFREG)
 		file->size = archive_entry_size(entry);
@@ -1677,10 +1685,10 @@ compression_init_encoder_deflate(struct archive *a,
 	 * of ugly hackery to convert a const * pointer to
 	 * a non-const pointer. */
 	strm->next_in = (Bytef *)(uintptr_t)(const void *)lastrm->next_in;
-	strm->avail_in = lastrm->avail_in;
+	strm->avail_in = (uInt)lastrm->avail_in;
 	strm->total_in = (uLong)lastrm->total_in;
 	strm->next_out = lastrm->next_out;
-	strm->avail_out = lastrm->avail_out;
+	strm->avail_out = (uInt)lastrm->avail_out;
 	strm->total_out = (uLong)lastrm->total_out;
 	if (deflateInit2(strm, level, Z_DEFLATED,
 	    (withheader)?15:-15,
@@ -1710,10 +1718,10 @@ compression_code_deflate(struct archive *a,
 	 * of ugly hackery to convert a const * pointer to
 	 * a non-const pointer. */
 	strm->next_in = (Bytef *)(uintptr_t)(const void *)lastrm->next_in;
-	strm->avail_in = lastrm->avail_in;
+	strm->avail_in = (uInt)lastrm->avail_in;
 	strm->total_in = (uLong)lastrm->total_in;
 	strm->next_out = lastrm->next_out;
-	strm->avail_out = lastrm->avail_out;
+	strm->avail_out = (uInt)lastrm->avail_out;
 	strm->total_out = (uLong)lastrm->total_out;
 	r = deflate(strm,
 	    (action == ARCHIVE_Z_FINISH)? Z_FINISH: Z_NO_FLUSH);
