@@ -661,13 +661,13 @@ static void verify(unsigned char *d, size_t s,
 	assertA(0 == archive_read_support_format_all(a));
 	assertA(0 == archive_read_open_memory(a, buff, s + 1024));
 	assertA(0 == archive_read_next_header(a, &ae));
-	assertEqualInt(archive_compression(a), ARCHIVE_COMPRESSION_NONE);
+	assertEqualInt(archive_filter_code(a, 0), ARCHIVE_FILTER_NONE);
 	assertEqualInt(archive_format(a), ARCHIVE_FORMAT_XAR);
 	/* Verify the only entry. */
 	f1(a, ae);
 	if (f2) {
 		assertA(0 == archive_read_next_header(a, &ae));
-		assertEqualInt(archive_compression(a), ARCHIVE_COMPRESSION_NONE);
+		assertEqualInt(archive_filter_code(a, 0), ARCHIVE_FILTER_NONE);
 		assertEqualInt(archive_format(a), ARCHIVE_FORMAT_XAR);
 		/* Verify the only entry. */
 		f2(a, ae);
