@@ -136,7 +136,7 @@ DEFINE_TEST(test_write_format_zip)
 #endif
 	assertEqualIntA(a, ARCHIVE_OK,
 	    archive_write_set_format_option(a, "zip", "compression", compression_type));
-	assertEqualIntA(a, ARCHIVE_OK, archive_write_set_compression_none(a));
+	assertEqualIntA(a, ARCHIVE_OK, archive_write_add_filter_none(a));
 	assertEqualIntA(a, ARCHIVE_OK,
 	    archive_write_open_memory(a, buff, buffsize, &used));
 
