@@ -2,27 +2,47 @@ DEFINE_TEST(test_0)
 DEFINE_TEST(test_basic)
 DEFINE_TEST(test_copy)
 DEFINE_TEST(test_empty_mtree)
+DEFINE_TEST(test_extract_tar_bz2)
+DEFINE_TEST(test_extract_tar_grz)
+DEFINE_TEST(test_extract_tar_gz)
+DEFINE_TEST(test_extract_tar_lrz)
+DEFINE_TEST(test_extract_tar_lz)
+DEFINE_TEST(test_extract_tar_lzma)
+DEFINE_TEST(test_extract_tar_lzo)
+DEFINE_TEST(test_extract_tar_xz)
+DEFINE_TEST(test_extract_tar_Z)
 DEFINE_TEST(test_format_newc)
 DEFINE_TEST(test_help)
-DEFINE_TEST(test_option_C_upper)
-DEFINE_TEST(test_option_H_upper)
-DEFINE_TEST(test_option_L_upper)
-DEFINE_TEST(test_option_O_upper)
-DEFINE_TEST(test_option_T_upper)
-DEFINE_TEST(test_option_U_upper)
-DEFINE_TEST(test_option_X_upper)
+DEFINE_TEST(test_option_a)
+DEFINE_TEST(test_option_b64encode)
 DEFINE_TEST(test_option_b)
+DEFINE_TEST(test_option_C_upper)
 DEFINE_TEST(test_option_exclude)
 DEFINE_TEST(test_option_gid_gname)
+DEFINE_TEST(test_option_grzip)
+DEFINE_TEST(test_option_H_upper)
+DEFINE_TEST(test_option_j)
 DEFINE_TEST(test_option_k)
 DEFINE_TEST(test_option_keep_newer_files)
+DEFINE_TEST(test_option_lrzip)
+DEFINE_TEST(test_option_L_upper)
+DEFINE_TEST(test_option_lzma)
+DEFINE_TEST(test_option_lzop)
 DEFINE_TEST(test_option_n)
 DEFINE_TEST(test_option_newer_than)
 DEFINE_TEST(test_option_nodump)
+DEFINE_TEST(test_option_older_than)
+DEFINE_TEST(test_option_O_upper)
 DEFINE_TEST(test_option_q)
 DEFINE_TEST(test_option_r)
 DEFINE_TEST(test_option_s)
+DEFINE_TEST(test_option_T_upper)
 DEFINE_TEST(test_option_uid_uname)
+DEFINE_TEST(test_option_uuencode)
+DEFINE_TEST(test_option_U_upper)
+DEFINE_TEST(test_option_X_upper)
+DEFINE_TEST(test_option_xz)
+DEFINE_TEST(test_option_z)
 DEFINE_TEST(test_patterns)
 DEFINE_TEST(test_print_longpath)
 DEFINE_TEST(test_stdio)
